@@ -1,7 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
-// taskwarrior - a command line task list manager.
 //
-// Copyright 2006-2013, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2015, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -27,18 +26,37 @@
 
 #ifndef INCLUDED_TASK
 #define INCLUDED_TASK
-#define L10N                                           // Localization complete.
 
 #include <vector>
 #include <map>
 #include <string>
 #include <stdio.h>
-
-
-void initializeUrgencyCoefficients ();
+#include <time.h>
 
 class Task : public std::map <std::string, std::string>
 {
+public:
+  static std::string defaultProject;
+  static std::string defaultDue;
+  static bool searchCaseSensitive;
+  static bool regex;
+  static std::map <std::string, std::string> attributes;  // name -> type
+  static std::map <std::string, float> coefficients;
+  static std::map <std::string, std::vector <std::string>> customOrder;
+  static float urgencyProjectCoefficient;
+  static float urgencyActiveCoefficient;
+  static float urgencyScheduledCoefficient;
+  static float urgencyWaitingCoefficient;
+  static float urgencyBlockedCoefficient;
+  static float urgencyInheritCoefficient;
+  static float urgencyAnnotationsCoefficient;
+  static float urgencyTagsCoefficient;
+  static float urgencyNextCoefficient;
+  static float urgencyDueCoefficient;
+  static float urgencyBlockingCoefficient;
+  static float urgencyAgeCoefficient;
+  static float urgencyAgeMax;
+
 public:
   Task ();                       // Default constructor
   Task (const Task&);            // Copy constructor
@@ -49,11 +67,14 @@ public:
 
   void parse (const std::string&);
   std::string composeF4 () const;
-  std::string composeJSON (bool include_id = false) const;
+  std::string composeJSON (bool decorate = false) const;
 
   // Status values.
   enum status {pending, completed, deleted, recurring, waiting};
 
+  // Date state values.
+  enum dateState {dateNotDue, dateAfterToday, dateLaterToday, dateEarlierToday, dateBeforeToday};
+
   // Public data.
   int id;
   float urgency_value;
@@ -68,29 +89,36 @@ public:
   static status textToStatus (const std::string&);
   static std::string statusToText (status);
 
-  void setEntry ();
-  void setEnd ();
-  void setStart ();
-  void setModified ();
-
+  void setAsNow (const std::string&);
   bool has (const std::string&) const;
   std::vector <std::string> all ();
   const std::string get (const std::string&) const;
   const std::string& get_ref (const std::string&) const;
   int get_int (const std::string&) const;
   unsigned long get_ulong (const std::string&) const;
+  float get_float (const std::string&) const;
   time_t get_date (const std::string&) const;
   void set (const std::string&, const std::string&);
   void set (const std::string&, int);
   void remove (const std::string&);
 
+#ifdef PRODUCT_TASKWARRIOR
+  bool is_ready () const;
   bool is_due () const;
+  bool is_dueyesterday () const;
   bool is_duetoday () const;
+  bool is_duetomorrow () const;
+  bool is_dueweek () const;
+  bool is_duemonth () const;
+  bool is_dueyear () const;
   bool is_overdue () const;
+#endif
 
   status getStatus () const;
   void setStatus (status);
 
+  dateState getDateState (const std::string&) const;
+
   int getTagCount () const;
   bool hasTag (const std::string&) const;
   void addTag (const std::string&);
@@ -104,6 +132,7 @@ public:
   void addAnnotation (const std::string&);
   void removeAnnotations ();
 
+#ifdef PRODUCT_TASKWARRIOR
   void addDependency (int);
   void addDependency (const std::string&);
   void removeDependency (int);
@@ -115,29 +144,37 @@ public:
   void getUDAOrphans (std::vector <std::string>&) const;
 
   void substitute (const std::string&, const std::string&, bool);
+#endif
 
   void validate (bool applyDefault = true);
 
   float urgency_c () const;
   float urgency ();
 
+  enum modType {modReplace, modPrepend, modAppend, modAnnotate};
+  void modify (modType, bool text_required = false);
+
 private:
   int determineVersion (const std::string&);
-  void legacyParse (const std::string&);
+  void parseJSON (const std::string&);
+  void parseLegacy (const std::string&);
   void validate_before (const std::string&, const std::string&);
+  const std::string encode (const std::string&) const;
+  const std::string decode (const std::string&) const;
 
-  inline float urgency_priority () const;
-  inline float urgency_project () const;
-  inline float urgency_active () const;
-  inline float urgency_scheduled () const;
-  inline float urgency_waiting () const;
-  inline float urgency_blocked () const;
-  inline float urgency_annotations () const;
-  inline float urgency_tags () const;
-  inline float urgency_next () const;
-  inline float urgency_due () const;
-  inline float urgency_blocking () const;
-  inline float urgency_age () const;
+public:
+  float urgency_project () const;
+  float urgency_active () const;
+  float urgency_scheduled () const;
+  float urgency_waiting () const;
+  float urgency_blocked () const;
+  float urgency_inherit () const;
+  float urgency_annotations () const;
+  float urgency_tags () const;
+  float urgency_next () const;
+  float urgency_due () const;
+  float urgency_blocking () const;
+  float urgency_age () const;
 };
 
 #endif
