@@ -1,7 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
-// taskwarrior - a command line task list manager.
 //
-// Copyright 2006-2014, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2015, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -47,6 +46,107 @@ CmdConfig::CmdConfig ()
   _displays_id = false;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+bool CmdConfig::setConfigVariable (std::string name, std::string value, bool confirmation /* = false */)
+{
+  // Read .taskrc (or equivalent)
+  std::vector <std::string> contents;
+  File::read (context.config._original_file, contents);
+
+  bool found = false;
+  bool change = false;
+
+  std::vector <std::string>::iterator line;
+  for (line = contents.begin (); line != contents.end (); ++line)
+  {
+    // If there is a comment on the line, it must follow the pattern.
+    std::string::size_type comment = line->find ("#");
+    std::string::size_type pos     = line->find (name + "=");
+
+    if (pos != std::string::npos &&
+        (comment == std::string::npos ||
+         comment > pos))
+    {
+      found = true;
+      if (!confirmation ||
+          confirm (format (STRING_CMD_CONFIG_CONFIRM, name, context.config.get (name), value)))
+      {
+        if (comment != std::string::npos)
+          *line = name + "=" + json::encode (value) + " " + line->substr (comment);
+        else
+          *line = name + "=" + json::encode (value);
+
+        change = true;
+      }
+    }
+  }
+
+  // Not found, so append instead.
+  if (!found &&
+      (!confirmation ||
+       confirm (format (STRING_CMD_CONFIG_CONFIRM2, name, value))))
+  {
+    contents.push_back (name + "=" + json::encode (value));
+    change = true;
+  }
+
+  if (change)
+    File::write (context.config._original_file, contents);
+
+  return change;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+int CmdConfig::unsetConfigVariable (std::string name, bool confirmation /* = false */)
+{
+  // Read .taskrc (or equivalent)
+  std::vector <std::string> contents;
+  File::read (context.config._original_file, contents);
+
+  bool found = false;
+  bool change = false;
+
+  std::vector <std::string>::iterator line;
+  for (line = contents.begin (); line != contents.end (); )
+  {
+    bool lineDeleted = false;
+
+    // If there is a comment on the line, it must follow the pattern.
+    std::string::size_type comment = line->find ("#");
+    std::string::size_type pos     = line->find (name + "=");
+
+    if (pos != std::string::npos &&
+        (comment == std::string::npos ||
+         comment > pos))
+    {
+      found = true;
+
+      // Remove name
+      if (!confirmation ||
+          confirm (format (STRING_CMD_CONFIG_CONFIRM3, name)))
+      {
+        // vector::erase method returns a valid iterator to the next object
+        line = contents.erase (line);
+        lineDeleted = true;
+        change = true;
+      }
+    }
+
+    if (! lineDeleted)
+      line++;
+  }
+
+  if (change)
+    File::write (context.config._original_file, contents);
+
+  if ( change && found )
+    return 0;
+  else if ( found )
+    return 1;
+  else
+    return 2;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 int CmdConfig::execute (std::string& output)
 {
@@ -54,7 +154,7 @@ int CmdConfig::execute (std::string& output)
   std::stringstream out;
 
   // Get the non-attribute, non-fancy command line arguments.
-  std::vector <std::string> words = context.a3.extract_words ();
+  std::vector <std::string> words = context.cli.getWords ();
 
   // Support:
   //   task config name value    # set name to value
@@ -62,9 +162,13 @@ int CmdConfig::execute (std::string& output)
   //   task config name          # remove name
   if (words.size ())
   {
+    bool confirmation = context.config.getBoolean ("confirmation");
+    bool found = false;
+
     std::string name = words[0];
     std::string value = "";
 
+    // Join the remaining words into config variable's value
     if (words.size () > 1)
     {
       for (unsigned int i = 1; i < words.size (); ++i)
@@ -80,82 +184,30 @@ int CmdConfig::execute (std::string& output)
     {
       bool change = false;
 
-      // Read .taskrc (or equivalent)
-      std::vector <std::string> contents;
-      File::read (context.config._original_file, contents);
-
       // task config name value
       // task config name ""
       if (words.size () > 1)
-      {
-        bool found = false;
-        std::vector <std::string>::iterator line;
-        for (line = contents.begin (); line != contents.end (); ++line)
-        {
-          // If there is a comment on the line, it must follow the pattern.
-          std::string::size_type comment = line->find ("#");
-          std::string::size_type pos     = line->find (name + "=");
-
-          if (pos != std::string::npos &&
-              (comment == std::string::npos ||
-               comment > pos))
-          {
-            found = true;
-            if (confirm (format (STRING_CMD_CONFIG_CONFIRM, name, context.config.get (name), value)))
-            {
-              if (comment != std::string::npos)
-                *line = name + "=" + json::encode (value) + " " + line->substr (comment);
-              else
-                *line = name + "=" + json::encode (value);
-
-              change = true;
-            }
-          }
-        }
-
-        // Not found, so append instead.
-        if (!found &&
-            confirm (format (STRING_CMD_CONFIG_CONFIRM2, name, value)))
-        {
-          contents.push_back (name + "=" + json::encode (value));
-          change = true;
-        }
-      }
+        change = setConfigVariable(name, value, confirmation);
 
       // task config name
       else
       {
-        bool found = false;
-        std::vector <std::string>::iterator line;
-        for (line = contents.begin (); line != contents.end (); ++line)
+        rc = unsetConfigVariable(name, confirmation);
+        if (rc == 0)
         {
-          // If there is a comment on the line, it must follow the pattern.
-          std::string::size_type comment = line->find ("#");
-          std::string::size_type pos     = line->find (name + "=");
-
-          if (pos != std::string::npos &&
-              (comment == std::string::npos ||
-               comment > pos))
-          {
-            found = true;
-
-            // Remove name
-            if (confirm (format (STRING_CMD_CONFIG_CONFIRM3, name)))
-            {
-              *line = "";
-              change = true;
-            }
-          }
+          change = true;
+          found = true;
         }
+        else if (rc == 1)
+          found = true;
 
         if (!found)
           throw format (STRING_CMD_CONFIG_NO_ENTRY, name);
       }
 
-      // Write .taskrc (or equivalent)
+      // Show feedback depending on whether .taskrc has been rewritten
       if (change)
       {
-        File::write (context.config._original_file, contents);
         out << format (STRING_CMD_CONFIG_FILE_MOD,
                        context.config._original_file._data)
             << "\n";
