@@ -1,7 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
-// taskwarrior - a command line task list manager.
 //
-// Copyright 2006-2013, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2015, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -25,19 +24,25 @@
 //
 ////////////////////////////////////////////////////////////////////////////////
 
-#define L10N                                           // Localization complete.
-
+#include <cmake.h>
 #include <iomanip>
 #include <sstream>
 #include <algorithm>
 #include <stdlib.h>
+#include <time.h>
 #include <RX.h>
 #include <Context.h>
 #include <i18n.h>
 #include <text.h>
 #include <util.h>
-#include <cmake.h>
+#ifdef HAVE_COMMIT
 #include <commit.h>
+#endif
+
+#ifdef HAVE_LIBGNUTLS
+#include <gnutls/gnutls.h>
+#endif
+
 #include <CmdDiagnostics.h>
 
 extern Context context;
@@ -59,14 +64,16 @@ CmdDiagnostics::CmdDiagnostics ()
 // kind of questions we always have to ask whenever something is wrong.
 int CmdDiagnostics::execute (std::string& output)
 {
-  Color bold ("bold");
+  Color bold;
+  if (context.color ())
+    bold = Color ("bold");
 
   std::stringstream out;
   out << "\n"
       << bold.colorize (PACKAGE_STRING)
       << "\n";
 
-  out << "  " << STRING_CMD_DIAG_PLATFORM << ": "
+  out << "   " << STRING_CMD_DIAG_PLATFORM << ": "
       <<
 #if defined (DARWIN)
          "Darwin"
@@ -84,6 +91,10 @@ int CmdDiagnostics::execute (std::string& output)
          "NetBSD"
 #elif defined (LINUX)
          "Linux"
+#elif defined (KFREEBSD)
+         "GNU/kFreeBSD"
+#elif defined (GNUHURD)
+         "GNU/Hurd"
 #else
          STRING_CMD_DIAG_UNKNOWN
 #endif
@@ -93,10 +104,10 @@ int CmdDiagnostics::execute (std::string& output)
   out << bold.colorize (STRING_CMD_DIAG_COMPILER)
       << "\n"
 #ifdef __VERSION__
-      << "   " << STRING_CMD_DIAG_VERSION << ": "
+      << "    " << STRING_CMD_DIAG_VERSION << ": "
       << __VERSION__ << "\n"
 #endif
-      << "      " << STRING_CMD_DIAG_CAPS << ":"
+      << "       " << STRING_CMD_DIAG_CAPS << ":"
 #ifdef __STDC__
       << " +stdc"
 #endif
@@ -118,58 +129,71 @@ int CmdDiagnostics::execute (std::string& output)
 #ifdef _LP64
       << " +LP64"
 #endif
-      << " +c" << sizeof (char)
-      << " +i" << sizeof (int)
-      << " +l" << sizeof (long)
-      << " +vp" << sizeof (void*)
+      << " +c"      << 8 * sizeof (char)
+      << " +i"      << 8 * sizeof (int)
+      << " +l"      << 8 * sizeof (long)
+      << " +vp"     << 8 * sizeof (void*)
+      << " +time_t" << 8 * sizeof (time_t)
+      << "\n";
+
+  // Compiler compliance level.
+  std::string compliance = "non-compliant";
+#ifdef __cplusplus
+  int level = __cplusplus;
+  if (level == 199711)
+    compliance = "C++98/03";
+  else if (level == 201103)
+    compliance = "C++11";
+  else
+    compliance = format (level);
+#endif
+  out << " " << STRING_CMD_DIAG_COMPLIANCE
+      << ": "
+      << compliance
       << "\n\n";
 
   out << bold.colorize (STRING_CMD_DIAG_FEATURES)
       << "\n"
 
   // Build date.
-      << "     " << STRING_CMD_DIAG_BUILT << ": " << __DATE__ << " " << __TIME__ << "\n"
-      << "    " << STRING_CMD_DIAG_COMMIT << ": " << COMMIT << "\n"
-      << "     CMake: " << CMAKE_VERSION << "\n"
-      << "      " << STRING_CMD_DIAG_CAPS << ":"
-#ifdef HAVE_LIBPTHREAD
-      << " +pthreads"
-#else
-      << " -pthreads"
+      << "      " << STRING_CMD_DIAG_BUILT << ": " << __DATE__ << " " << __TIME__ << "\n"
+#ifdef HAVE_COMMIT
+      << "     " << STRING_CMD_DIAG_COMMIT << ": " << COMMIT << "\n"
 #endif
+      << "      CMake: " << CMAKE_VERSION << "\n";
 
-#ifdef HAVE_SRANDOM
-      << " +srandom"
+  out << "    libuuid: "
+#ifdef HAVE_UUID_UNPARSE_LOWER
+      << "libuuid + uuid_unparse_lower"
 #else
-      << " -srandom"
-#endif
-
-#ifdef HAVE_RANDOM
-      << " +random"
-#else
-      << " -random"
-#endif
-
-#ifdef HAVE_UUID
-      << " +uuid"
-#else
-      << " -uuid"
+      << "libuuid, no uuid_unparse_lower"
 #endif
       << "\n";
-  out << "   libuuid: "
-#if defined (HAVE_UUID) and defined (HAVE_UUID_UNPARSE_LOWER)
-      << "libuuid + uuid_unparse_lower"
-#elif defined (HAVE_UUID) and !defined (HAVE_UUID_UNPARSE_LOWER)
-      << "libuuid, no uuid_unparse_lower"
+
+  out << "  libgnutls: "
+#ifdef HAVE_LIBGNUTLS
+#ifdef GNUTLS_VERSION
+      << GNUTLS_VERSION
+#elif defined LIBGNUTLS_VERSION
+      << LIBGNUTLS_VERSION
+#endif
 #else
       << "n/a"
+#endif
+      << "\n";
+
+  out << " Build type: "
+#ifdef CMAKE_BUILD_TYPE
+      << CMAKE_BUILD_TYPE
+#else
+      << "-"
 #endif
       << "\n\n";
 
   // Config: .taskrc found, readable, writable
   out << bold.colorize (STRING_CMD_DIAG_CONFIG)
       << "\n"
-      << "      File: " << context.config._original_file._data << " "
+      << "       File: " << context.config._original_file._data << " "
       << (context.config._original_file.exists ()
            ? STRING_CMD_DIAG_FOUND
            : STRING_CMD_DIAG_MISSING)
@@ -181,7 +205,7 @@ int CmdDiagnostics::execute (std::string& output)
 
   // Config: data.location found, readable, writable
   File location (context.config.get ("data.location"));
-  out << "      Data: " << location._data << " "
+  out << "       Data: " << location._data << " "
       << (location.exists ()
            ? STRING_CMD_DIAG_FOUND
            : STRING_CMD_DIAG_MISSING)
@@ -191,12 +215,26 @@ int CmdDiagnostics::execute (std::string& output)
       << location.mode ()
       << "\n";
 
-  out << "    Server: "
-      << context.config.get ("taskd.server")
+  char* env = getenv ("TASKRC");
+  if (env)
+    out << "     TASKRC: "
+        << env
+        << "\n";
+
+  env = getenv ("TASKDATA");
+  if (env)
+    out << "   TASKDATA: "
+        << env
+        << "\n";
+
+  out << "    Locking: "
+      << (context.config.getBoolean ("locking")
+           ? STRING_CMD_DIAG_ENABLED
+           : STRING_CMD_DIAG_DISABLED)
       << "\n";
 
-  out << "   Locking: "
-      << (context.config.getBoolean ("locking")
+  out << "         GC: "
+      << (context.config.getBoolean ("gc")
            ? STRING_CMD_DIAG_ENABLED
            : STRING_CMD_DIAG_DISABLED)
       << "\n";
@@ -204,97 +242,97 @@ int CmdDiagnostics::execute (std::string& output)
   // Determine rc.editor/$EDITOR/$VISUAL.
   char* peditor;
   if (context.config.get ("editor") != "")
-    out << " rc.editor: " << context.config.get ("editor") << "\n";
+    out << "  rc.editor: " << context.config.get ("editor") << "\n";
   else if ((peditor = getenv ("VISUAL")) != NULL)
-    out << "   $VISUAL: " << peditor << "\n";
+    out << "    $VISUAL: " << peditor << "\n";
   else if ((peditor = getenv ("EDITOR")) != NULL)
-    out << "   $EDITOR: " << peditor << "\n";
+    out << "    $EDITOR: " << peditor << "\n";
+
+  out << "     Server: "
+      << context.config.get ("taskd.server")
+      << "\n";
+
+  if (context.config.get ("taskd.ca") != "")
+    out << "         CA: "
+        << context.config.get ("taskd.ca")
+        << (File (context.config.get ("taskd.ca")).readable ()
+             ? " (readable)" : " (not readable)")
+        << "\n";
+
+  std::string trust_value = context.config.get ("taskd.trust");
+  if (trust_value == "strict" ||
+      trust_value == "ignore hostname" ||
+      trust_value == "allow all")
+    out << "      Trust: " << trust_value << "\n";
+  else
+    out << "      Trust: Bad value - see 'man taskrc'\n";
+
+  out << "Certificate: "
+      << context.config.get ("taskd.certificate")
+      << (File (context.config.get ("taskd.certificate")).readable ()
+           ? " (readable)" : " (not readable)")
+      << "\n";
+
+  out << "        Key: "
+      << context.config.get ("taskd.key")
+      << (File (context.config.get ("taskd.key")).readable ()
+           ? " (readable)" : " (not readable)")
+      << "\n";
+
+  out << "    Ciphers: "
+      << context.config.get ("taskd.ciphers")
+      << "\n";
+
+  // Get credentials, but mask out the key.
+  std::string credentials = context.config.get ("taskd.credentials");
+  std::string::size_type last_slash = credentials.rfind ('/');
+  if (last_slash != std::string::npos)
+    credentials = credentials.substr (0, last_slash)
+                + "/"
+                + std::string (credentials.length () - last_slash - 1, '*');
+
+  out << "      Creds: "
+      << credentials
+      << "\n\n";
+
+  // Disaply hook status.
+  out << bold.colorize (STRING_CMD_DIAG_HOOKS)
+      << "\n"
+      << "    Scripts: "
+      << (context.config.getBoolean ("hooks") ? STRING_CMD_DIAG_HOOK_ENABLE : STRING_CMD_DIAG_HOOK_DISABLE)
+      << "\n";
+
+  std::vector <std::string> hooks = context.hooks.list ();
+  if (hooks.size ())
+  {
+    std::vector <std::string>::iterator h;
+    for (h = hooks.begin (); h != hooks.end (); ++h)
+    {
+      Path p (*h);
+      std::string name = p.name ();
+      out << "             "
+          << *h
+          << (p.executable () ? format (" ({1})", STRING_CMD_DIAG_HOOK_EXEC) : format (" ({1})", STRING_CMD_DIAG_HOOK_NO_EXEC))
+          << (p.is_link () ? format (" ({1})", STRING_CMD_DIAG_HOOK_SYMLINK) : "")
+          << ((name.substr (0, 6) == "on-add" ||
+               name.substr (0, 9) == "on-modify" ||
+               name.substr (0, 9) == "on-launch" ||
+               name.substr (0, 7) == "on-exit") ? "" : format (" ({1})", STRING_CMD_DIAG_HOOK_NAME))
+          << "\n";
+    }
+  }
+  else
+    out << format ("             ({1})\n", STRING_CMD_DIAG_NONE);
 
   out << "\n";
 
-  // External commands.
-  out << bold.colorize (STRING_CMD_DIAG_EXTERNAL)
-      << "\n";
-  {
-    std::vector <std::string> matches;
-    char buffer [1024] = {0};
-    FILE* fp;
-    if ((fp = popen ("/usr/bin/env scp 2>&1", "r")))
-    {
-      char* p = fgets (buffer, 1023, fp);
-      pclose (fp);
-
-      RX r ("usage", false);
-      if (p)
-        out << "       scp: "
-            << (r.match (buffer)
-                 ? STRING_CMD_DIAG_FOUND
-                 : STRING_CMD_DIAG_MISSING)
-            << "\n";
-    }
-
-    if ((fp = popen ("/usr/bin/env rsync --version 2>&1", "r")))
-    {
-      char* p = fgets (buffer, 1023, fp);
-      pclose (fp);
-
-      // rsync  version 2.6.9  protocol version 29
-      if (p)
-      {
-        RX r ("version ([0-9]+\\.[0-9]+\\.[0-9]+)", false);
-        matches.clear ();
-        r.match (matches, buffer);
-        out << "     rsync: "
-            << (matches.size () ? matches[0] : STRING_CMD_DIAG_MISSING)
-            << "\n";
-      }
-    }
-
-    if ((fp = popen ("/usr/bin/env curl --version 2>&1", "r")))
-    {
-      char* p = fgets (buffer, 1023, fp);
-      pclose (fp);
-
-      // curl 7.19.7 (universal-apple-darwin10.0) libcurl/7.19.7 OpenSSL/0.9.8l zlib/1.2.3
-      if (p)
-      {
-        RX r ("curl ([0-9]+\\.[0-9]+\\.[0-9]+)", false);
-        matches.clear ();
-        r.match (matches, buffer);
-        out << "      curl: "
-            << (matches.size () ? matches[0] :  STRING_CMD_DIAG_MISSING)
-            << "\n";
-      }
-    }
-
-    out << "\n";
-  }
-
-  // Generate 1000 UUIDs and verify they are all unique.
+  // Verify UUIDs are all unique.
   out << bold.colorize (STRING_CMD_DIAG_TESTS)
       << "\n";
   {
-    out << "  UUID gen: ";
-    std::vector <std::string> uuids;
-    std::string id;
-    for (int i = 0; i < 1000; i++)
-    {
-      id = uuid ();
-      if (std::find (uuids.begin (), uuids.end (), id) != uuids.end ())
-      {
-        out << format (STRING_CMD_DIAG_UUID_BAD, i) << "\n";
-        break;
-      }
-      else
-        uuids.push_back (id);
-    }
-
-    if (uuids.size () >= 1000)
-      out << STRING_CMD_DIAG_UUID_GOOD << "\n";
-
     // Determine terminal details.
     const char* term = getenv ("TERM");
-    out << "     $TERM: "
+    out << "      $TERM: "
         << (term ? term : STRING_CMD_DIAG_NONE)
         << " ("
         << context.getWidth ()
@@ -317,7 +355,7 @@ int CmdDiagnostics::execute (std::string& output)
          seen[uuid] = 0;
     }
 
-    out << "      Dups: "
+    out << "       Dups: "
         << format (STRING_CMD_DIAG_UUID_SCAN, all.size ())
         << "\n";
 
@@ -325,11 +363,11 @@ int CmdDiagnostics::execute (std::string& output)
     {
       std::vector <std::string>::iterator d;
       for (d = dups.begin (); d != dups.end (); ++d)
-        out << "            " << format (STRING_CMD_DIAG_UUID_DUP, *d) << "\n";
+        out << "             " << format (STRING_CMD_DIAG_UUID_DUP, *d) << "\n";
     }
     else
     {
-      out << "            " << STRING_CMD_DIAG_UUID_NO_DUP
+      out << "             " << STRING_CMD_DIAG_UUID_NO_DUP
           << "\n";
     }
   }
