@@ -1,7 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
-// taskwarrior - a command line task list manager.
 //
-// Copyright 2006-2013, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2015, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -25,43 +24,40 @@
 //
 ////////////////////////////////////////////////////////////////////////////////
 
-#define L10N                                           // Localization complete.
-
+#include <cmake.h>
+// If <iostream> is included, put it after <stdio.h>, because it includes
+// <stdio.h>, and therefore would ignore the _WITH_GETLINE.
+#ifdef FREEBSD
+#define _WITH_GETLINE
+#endif
+#include <stdio.h>
 #include <iostream>
 #include <fstream>
-#include <limits>
 #include <sstream>
 #include <vector>
 #include <algorithm>
+#include <cstring>
 #include <string>
 #include <stdint.h>
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/wait.h>
-#include <fcntl.h>
 #include <unistd.h>
-#include <math.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <pwd.h>
 #include <errno.h>
+#include <signal.h>
+#include <sys/select.h>
 
 #include <Date.h>
 #include <text.h>
 #include <main.h>
 #include <i18n.h>
 #include <util.h>
-#include <cmake.h>
 
 extern Context context;
 
-////////////////////////////////////////////////////////////////////////////////
-static inline unsigned round_up_to (unsigned n, unsigned target)
-{
-  return n + target - (n % target);
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 // Uses std::getline, because std::cin eats leading whitespace, and that means
 // that if a newline is entered, std::cin eats it and never returns from the
@@ -91,44 +87,6 @@ bool confirm (const std::string& question)
   return matches[0] == STRING_UTIL_CONFIRM_YES ? true : false;
 }
 
-////////////////////////////////////////////////////////////////////////////////
-// 0 = no
-// 1 = yes
-// 2 = all
-int confirm3 (const std::string& question)
-{
-  std::vector <std::string> options;
-  options.push_back (STRING_UTIL_CONFIRM_YES_U);
-  options.push_back (STRING_UTIL_CONFIRM_YES);
-  options.push_back (STRING_UTIL_CONFIRM_NO);
-  options.push_back (STRING_UTIL_CONFIRM_ALL_U);
-  options.push_back (STRING_UTIL_CONFIRM_ALL);
-
-  std::string answer;
-  std::vector <std::string> matches;
-
-  do
-  {
-    std::cout << question
-              << " ("
-              << options[1] << "/"
-              << options[2] << "/"
-              << options[4]
-              << ") ";
-
-    std::getline (std::cin, answer);
-    answer = trim (answer);
-    autoComplete (answer, options, matches, 1); // Hard-coded 1.
-  }
-  while (matches.size () != 1);
-
-       if (matches[0] == STRING_UTIL_CONFIRM_YES_U) return 1;
-  else if (matches[0] == STRING_UTIL_CONFIRM_YES)   return 1;
-  else if (matches[0] == STRING_UTIL_CONFIRM_ALL_U) return 2;
-  else if (matches[0] == STRING_UTIL_CONFIRM_ALL)   return 2;
-  else                                              return 0;
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 // 0 = no
 // 1 = yes
@@ -171,16 +129,6 @@ int confirm4 (const std::string& question)
   else                           return 0;
 }
 
-////////////////////////////////////////////////////////////////////////////////
-void delay (float f)
-{
-  struct timeval t;
-  t.tv_sec = (int) f;
-  t.tv_usec = int ((f - (int)f) * 1000000);
-
-  select (0, NULL, NULL, NULL, &t);
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 // Convert a quantity in seconds to a more readable format.
 std::string formatBytes (size_t bytes)
@@ -204,7 +152,7 @@ int autoComplete (
 {
   matches.clear ();
 
-  // Handle trivial case. 
+  // Handle trivial case.
   unsigned int length = partial.length ();
   if (length)
   {
@@ -231,9 +179,29 @@ int autoComplete (
   return matches.size ();
 }
 
-////////////////////////////////////////////////////////////////////////////////
-#ifdef HAVE_UUID
+// Handle the generation of UUIDs on FreeBSD in a separate implementation
+// of the uuid () function, since the API is quite different from Linux's.
+// Also, uuid_unparse_lower is not needed on FreeBSD, because the string
+// representation is always lowercase anyway.
+// For the implementation details, refer to
+// http://svnweb.freebsd.org/base/head/sys/kern/kern_uuid.c
+#ifdef FREEBSD
+const std::string uuid ()
+{
+  uuid_t id;
+  uint32_t status;
+  char *buffer (0);
+  uuid_create (&id, &status);
+  uuid_to_string (&id, &buffer, &status);
 
+  std::string res (buffer);
+  free (buffer);
+
+  return res;
+}
+#else
+
+////////////////////////////////////////////////////////////////////////////////
 #ifndef HAVE_UUID_UNPARSE_LOWER
 // Older versions of libuuid don't have uuid_unparse_lower(), only uuid_unparse()
 void uuid_unparse_lower (uuid_t uu, char *out)
@@ -257,311 +225,154 @@ const std::string uuid ()
 
   return std::string (buffer);
 }
-
-////////////////////////////////////////////////////////////////////////////////
-#else
-
-#ifdef HAVE_RANDOM
-#define rand() random()
 #endif
 
 ////////////////////////////////////////////////////////////////////////////////
-const std::string uuid ()
+// Run a binary with args, capturing output.
+int execute (
+  const std::string& executable,
+  const std::vector <std::string>& args,
+  const std::string& input,
+  std::string& output)
 {
-  uint32_t time_low = ((rand () << 16) & 0xffff0000) | (rand () & 0xffff);
-  uint16_t time_mid = rand () & 0xffff;
-  uint16_t time_high_and_version = (rand () & 0x0fff) | 0x4000;
-  uint16_t clock_seq = (rand () & 0x3fff) | 0x8000;
-  uint8_t node [6];
-  for (size_t i = 0; i < 6; i++)
-    node[i] = rand() & 0xff;
+  pid_t pid;
+  int pin[2], pout[2];
+  fd_set rfds, wfds;
+  struct timeval tv;
+  int select_retval, read_retval, write_retval;
+  char buf[16384];
+  unsigned int written;
+  const char* input_cstr = input.c_str ();
 
-  char buffer[37];
-  sprintf(buffer, "%08x-%04x-%04x-%02x%02x-%02x%02x%02x%02x%02x%02x",
-    time_low, time_mid, time_high_and_version, clock_seq >> 8, clock_seq & 0xff,
-    node[0], node[1], node[2], node[3], node[4], node[5]);
+  if (signal (SIGPIPE, SIG_IGN) == SIG_ERR) // Handled locally with EPIPE.
+    throw std::string (std::strerror (errno));
 
-  return std::string (buffer);
-}
-#endif
+  if (pipe (pin) == -1)
+    throw std::string (std::strerror (errno));
 
-////////////////////////////////////////////////////////////////////////////////
-// On Solaris no flock function exists.
-#ifdef SOLARIS
-int flock (int fd, int operation)
-{
-  struct flock fl;
+  if (pipe (pout) == -1)
+    throw std::string (std::strerror (errno));
 
-  switch (operation & ~LOCK_NB)
+  if ((pid = fork ()) == -1)
+    throw std::string (std::strerror (errno));
+
+  if (pid == 0)
   {
-  case LOCK_SH:
-    fl.l_type = F_RDLCK;
-    break;
+    // This is only reached in the child
+    close (pin[1]);   // Close the write end of the input pipe.
+    close (pout[0]);  // Close the read end of the output pipe.
 
-  case LOCK_EX:
-    fl.l_type = F_WRLCK;
-    break;
+    if (dup2 (pin[0], STDIN_FILENO) == -1)
+      throw std::string (std::strerror (errno));
+    close (pin[0]);
 
-  case LOCK_UN:
-    fl.l_type = F_UNLCK;
-    break;
+    if (dup2 (pout[1], STDOUT_FILENO) == -1)
+      throw std::string (std::strerror (errno));
+    close (pout[1]);
 
-  default:
-    errno = EINVAL;
-    return -1;
+    char** argv = new char* [args.size () + 2];
+    argv[0] = (char*) executable.c_str ();
+    for (unsigned int i = 0; i < args.size (); ++i)
+      argv[i+1] = (char*) args[i].c_str ();
+
+    argv[args.size () + 1] = NULL;
+    _exit (execvp (executable.c_str (), argv));
   }
 
-  fl.l_whence = 0;
-  fl.l_start  = 0;
-  fl.l_len    = 0;
+  // This is only reached in the parent
+  close (pin[0]);   // Close the read end of the input pipe.
+  close (pout[1]);  // Close the write end of the output pipe.
 
-  return fcntl (fd, (operation & LOCK_NB) ? F_SETLK : F_SETLKW, &fl);
-}
-#endif
-
-////////////////////////////////////////////////////////////////////////////////
-// The vector must be sorted first.  This is a modified version of the run-
-// length encoding algorithm.
-//
-// This function converts the vector:
-//
-//   [1, 3, 4, 6, 7, 8, 9, 11]
-//
-// to ths string:
-//
-//   1,3-4,6-9,11
-//
-std::string compressIds (const std::vector <int>& ids)
-{
-  std::stringstream result;
-
-  int range_start = 0;
-  int range_end = 0;
-
-  for (unsigned int i = 0; i < ids.size (); ++i)
+  if (input.size () == 0)
   {
-    if (i + 1 == ids.size ())
-    {
-      if (result.str ().length ())
-        result << ",";
+    // Nothing to send to the child, close the pipe early.
+    close (pin[1]);
+  }
 
-      if (range_start < range_end)
-        result << ids[range_start] << "-" << ids[range_end];
-      else
-        result << ids[range_start];
+  read_retval = -1;
+  written = 0;
+  while (read_retval != 0 || input.size () != written)
+  {
+    FD_ZERO (&rfds);
+    if (read_retval != 0)
+    {
+      FD_SET (pout[0], &rfds);
     }
-    else
-    {
-      if (ids[range_end] + 1 == ids[i + 1])
-      {
-        ++range_end;
-      }
-      else
-      {
-        if (result.str ().length ())
-          result << ",";
 
-        if (range_start < range_end)
-          result << ids[range_start] << "-" << ids[range_end];
+    FD_ZERO (&wfds);
+    if (input.size () != written)
+    {
+      FD_SET (pin[1], &wfds);
+    }
+
+    tv.tv_sec = 5;
+    tv.tv_usec = 0;
+
+    select_retval = select (std::max (pout[0], pin[1]) + 1, &rfds, &wfds, NULL, &tv);
+
+    if (select_retval == -1)
+      throw std::string (std::strerror (errno));
+
+    if (FD_ISSET (pin[1], &wfds))
+    {
+      write_retval = write (pin[1], input_cstr + written, input.size () - written);
+      if (write_retval == -1)
+      {
+        if (errno == EPIPE)
+        {
+          // Child died (or closed the pipe) before reading all input.
+          // We don't really care; pretend we wrote it all.
+          write_retval = input.size () - written;
+        }
         else
-          result << ids[range_start];
-
-        range_start = range_end = i + 1;
+        {
+          throw std::string (std::strerror (errno));
+        }
       }
+      written += write_retval;
+
+      if (written == input.size ())
+      {
+        // Let the child know that no more input is coming by closing the pipe.
+        close (pin[1]);
+      }
+    }
+
+    if (FD_ISSET (pout[0], &rfds))
+    {
+      read_retval = read (pout[0], &buf, sizeof(buf)-1);
+      if (read_retval == -1)
+        throw std::string (std::strerror (errno));
+
+      buf[read_retval] = '\0';
+      output += buf;
     }
   }
 
-  return result.str ();
-}
+  close (pout[0]);  // Close the read end of the output pipe.
 
-////////////////////////////////////////////////////////////////////////////////
-void combine (std::vector <int>& dest, const std::vector <int>& source)
-{
-  // Create a map using the sequence elements as keys.  This will create a
-  // unique list, with no duplicates.
-  std::map <int, int> both;
-  std::vector <int>::iterator i1;
-  for (i1 = dest.begin (); i1 != dest.end (); ++i1) both[*i1] = 0;
+  int status = -1;
+  if (wait (&status) == -1)
+    throw std::string (std::strerror (errno));
 
-  std::vector <int>::const_iterator i2;
-  for (i2 = source.begin (); i2 != source.end (); ++i2) both[*i2] = 0;
-
-  // Now make a sequence out of the keys of the map.
-  dest.clear ();
-  std::map <int, int>::iterator i3;
-  for (i3 = both.begin (); i3 != both.end (); ++i3)
-    dest.push_back (i3->first);
-
-  std::sort (dest.begin (), dest.end ());
-}
-
-////////////////////////////////////////////////////////////////////////////////
-// Run an external executable with execvp. This means stdio goes to 
-// the child process, so that it can receive user input (e.g. passwords).
-//
-int execute(const std::string& executable, std::vector<std::string> arguments)
-{
-  if (executable == "")
-    return -1;
-
-  // create command line before forking because the parent process also needs this for 
-  // calling context.debug()
-  char shell[] = "sh";
-  char opt[]   = "-c";
-
-  std::string cmdline = executable;
-
-  std::vector <std::string>::iterator it;
-  for (it = arguments.begin(); it != arguments.end(); ++it)
+  if (WIFEXITED (status))
   {
-    cmdline += " " + (std::string)*it;
-  }
-
-  context.debug ("Executing: " + std::string(shell) + " " + std::string(opt) + " " + cmdline);
-
-  pid_t child_pid = fork();
-
-  if (child_pid == 0)
-  {
-    // this is done by the child process
-    char** argv = new char*[4];
-    argv[0] = shell;                  // sh
-    argv[1] = opt;                    // -c
-    argv[2] = (char*)cmdline.c_str(); // e.g. scp undo.data user@host:.task/
-    argv[3] = NULL;                   // required by execv
-
-    int ret = execvp(shell, argv);
-    delete[] argv;
-
-    exit(ret);
-  }
-  else if (child_pid == -1)
-  {
-    return -1;
+    status = WEXITSTATUS (status);
   }
   else
   {
-    // this is done by the parent process
-    int child_status;
-
-    pid_t pid = waitpid(child_pid, &child_status, 0);
-
-    if (pid == -1)
-      return -1;
-    else
-      return WEXITSTATUS (child_status);
+    throw std::string ("Error: Could not get Hook exit status!");
   }
+
+  if (signal (SIGPIPE, SIG_DFL) == SIG_ERR) // We're done, return to default.
+    throw std::string (std::strerror (errno));
+
+  return status;
 }
 
 // Collides with std::numeric_limits methods
 #undef max
 
-////////////////////////////////////////////////////////////////////////////////
-unsigned burndown_size (unsigned ntasks)
-{
-  // Nearest 2
-  if (ntasks < 20)
-    return round_up_to (ntasks, 2);
-
-  // Nearest 10
-  if (ntasks < 50)
-    return round_up_to (ntasks, 10);
-
-  // Nearest 20
-  if (ntasks < 100)
-    return round_up_to (ntasks, 20);
-
-  // Choose the number from here rounded up to the nearest 10% of the next
-  // highest power of 10 or half of power of 10.
-  const unsigned count = (unsigned) log10 (std::numeric_limits<unsigned>::max ());
-  unsigned half = 500;
-  unsigned full = 1000;
-
-  // We start at two because we handle 5, 10, 50, and 100 above.
-  for (unsigned i = 2; i < count; ++i)
-  {
-    if (ntasks < half)
-      return round_up_to (ntasks, half / 10);
-
-    if (ntasks < full)
-      return round_up_to (ntasks, full / 10);
-
-    half *= 10;
-    full *= 10;
-  }
-
-  // Round up to max of unsigned.
-  return std::numeric_limits<unsigned>::max ();
-}
-
-////////////////////////////////////////////////////////////////////////////////
-// Encode values prior to serialization.
-//   [  -> &open;
-//   ]  -> &close;
-const std::string encode (const std::string& value)
-{
-  std::string modified = value;
-
-  str_replace (modified, "[",  "&open;");
-  str_replace (modified, "]",  "&close;");
-
-  return modified;
-}
-
-////////////////////////////////////////////////////////////////////////////////
-// Decode values after parse.
-//   "  <- &dquot;
-//   '  <- &squot; or &quot;
-//   ,  <- &comma;
-//   [  <- &open;
-//   ]  <- &close;
-//   :  <- &colon;
-const std::string decode (const std::string& value)
-{
-  if (value.find ('&') != std::string::npos)
-  {
-    std::string modified = value;
-
-    // Supported encodings.
-    str_replace (modified, "&open;",  "[");
-    str_replace (modified, "&close;", "]");
-
-    // Support for deprecated encodings.  These cannot be removed or old files
-    // will not be parsable.  Not just old files - completed.data can contain
-    // tasks formatted/encoded using these.
-    str_replace (modified, "&dquot;", "\"");
-    str_replace (modified, "&quot;",  "'");
-    str_replace (modified, "&squot;", "'");  // Deprecated 2.0
-    str_replace (modified, "&comma;", ",");  // Deprecated 2.0
-    str_replace (modified, "&colon;", ":");  // Deprecated 2.0
-
-    return modified;
-  }
-
-  return value;
-}
-
-////////////////////////////////////////////////////////////////////////////////
-// Escapes any unescaped character of type c within the given string
-// e.g. ' ' -> '\ '
-const std::string escape (const std::string& value, char c)
-{
-  std::string modified = value;
-  char tmp[2] = {c, '\0'};
-  std::string search  = tmp;
-  std::string replace = "\\" + search;
-
-  std::string::size_type pos = modified.find (search);
-  while (pos != std::string::npos) {
-    if ( modified[pos-1] != '\\' )
-      modified.replace (pos, 1, replace);
-
-    pos = modified.find (search, pos+1);
-  }
-
-  return modified;
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 // Accept a list of projects, and return an indented list
 // that reflects the hierarchy.
@@ -584,20 +395,6 @@ const std::string escape (const std::string& value, char c)
 //  - delimiter is the character used to split up projects into subprojects.
 //    - defaults to the period, '.'
 //
-const std::vector<std::string> indentTree (
-  const std::vector<std::string>& values,
-  const std::string& whitespace /* = "  " */,
-  char delimiter/* = '.' */)
-{
-  std::vector <std::string> modified;
-  std::vector <std::string>::const_iterator i;
-  for (i = values.begin (); i != values.end (); ++i)
-    modified.push_back (indentProject (*i, whitespace, delimiter));
-
-  return modified;
-}
-
-////////////////////////////////////////////////////////////////////////////////
 const std::string indentProject (
   const std::string& project,
   const std::string& whitespace /* = "  " */,
